@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtXml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -38,6 +44,8 @@
 #include <qmap.h>
 #include <qxml.h>
 
+#include <stack>
+
 QT_BEGIN_NAMESPACE
 
 //
@@ -61,7 +69,7 @@ private:
     void initIncrementalParsing();
 
     // used to determine if elements are correctly nested
-    QStack<QString> tags;
+    std::stack<QString, QStringList> tags;
 
     // used by parseReference() and parsePEReference()
     enum EntityRecognitionContext { InContent, InAttributeValue, InEntityValue, InDTD };
@@ -77,6 +85,8 @@ private:
 
         Q_DUMMY_COMPARISON_OPERATOR(ExternParameterEntity)
     };
+    friend class QTypeInfo<ExternParameterEntity>;
+
     struct ExternEntity
     {
         ExternEntity() {}
@@ -87,6 +97,8 @@ private:
         QString notation;
         Q_DUMMY_COMPARISON_OPERATOR(ExternEntity)
     };
+    friend class QTypeInfo<ExternEntity>;
+
     QMap<QString,ExternParameterEntity> externParameterEntities;
     QMap<QString,QString> parameterEntities;
     QMap<QString,ExternEntity> externEntities;
@@ -104,6 +116,7 @@ private:
         QString value;
         int index;
     };
+    friend class QTypeInfo<XmlRef>;
     QStack<XmlRef> xmlRefStack;
 
     // used for standalone declaration
@@ -166,6 +179,7 @@ private:
         ParseFunction function;
         int state;
     };
+    friend class QTypeInfo<ParseState>;
     QStack<ParseState> *parseStack;
 
     // used in parseProlog()
@@ -215,7 +229,7 @@ private:
     // for the DTD currently being parsed.
     static const int dtdRecursionLimit = 2;
     // The maximum amount of characters an entity value may contain, after expansion.
-    static const int entityCharacterLimit = 65536;
+    static const int entityCharacterLimit = 4096;
 
     const QString &string();
     void stringClear();
@@ -294,6 +308,10 @@ private:
     friend class QXmlSimpleReaderLocator;
     friend class QDomHandler;
 };
+Q_DECLARE_TYPEINFO(QXmlSimpleReaderPrivate::ParseState, Q_PRIMITIVE_TYPE);
+Q_DECLARE_TYPEINFO(QXmlSimpleReaderPrivate::XmlRef, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(QXmlSimpleReaderPrivate::ExternParameterEntity, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(QXmlSimpleReaderPrivate::ExternEntity, Q_MOVABLE_TYPE);
 
 QT_END_NAMESPACE
 
